@@ -5,9 +5,9 @@ use strict;
 use warnings;
 no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
-our $VERSION = 0.06;
+our $VERSION = 0.07;
 
-my $self;
+my$self;
 
 $" = ', ';
 $| = 1;
@@ -25,13 +25,13 @@ sub new{
 		itemsKeyDescription => undef,
 		itemsKeyTransactions => undef,
 		quantityPossibleRules => undef,
-		frequentItemset => undef,
+		frequentItemset => [],
 		recursively => 0,
 		associationRules => undef,
-		itemsetSize => undef,
+		implications => undef,
+		itemsetSize => 2,
 		rule => 0
 	};
-	@{$self->{frequentItemset}}=();
 	bless($self,$class);
 	return $self;
 }
@@ -53,127 +53,125 @@ sub validate_data{
 
 sub generate_rules{
 	$self->validate_data;
-	my @itemsetCandidateSizeOne = keys(%{$self->{itemsKeyDescription}});
-	if($self->{messages}){
-		my $quantityItems = scalar(@itemsetCandidateSizeOne);
+	my @candidateItemsetSizeOne = $self->data_persistence;
+	if($self->{messages}&&!$self->{recursively}){
+		my $quantityItems = scalar(@candidateItemsetSizeOne);
 		$self->{quantityPossibleRules} = ((3**$quantityItems)-(2**($quantityItems+1))+1);
 		print "\n$quantityItems items, $self->{quantityPossibleRules} possible rules";
 	}
 	$self->{recursively} = 1;
-	$self->association_rules_itemset_size(\@itemsetCandidateSizeOne, 2);
+	$self->association_rules_itemset_size(\@candidateItemsetSizeOne);
 }
 
 sub association_rules_itemset_size{
-	my @itemsetCandidateSizeOne;
-	my $itemsetSizeLimit;
+	my @candidateItemsetSizeOne;
 	if($self->{recursively}){
-		@itemsetCandidateSizeOne = @{$_[1]};
-		$itemsetSizeLimit = $_[2];
-		$self->{itemsetSize} = $_[2];
+		@candidateItemsetSizeOne = @{$_[1]};
 	}
 	else{
 		$self->validate_data;
 		(scalar(@_)==2)
 			or die("Error: Invalid number of arguments! It is expected a number representing the size of itemset!\n");
-		@itemsetCandidateSizeOne = keys(%{$self->{itemsKeyDescription}});
-		((scalar(@itemsetCandidateSizeOne)>=$_[1])&&($_[1]>=2))
+		@candidateItemsetSizeOne = $self->data_persistence;
+		((scalar(@candidateItemsetSizeOne)>=$_[1])&&($_[1]>=2))
 			or die("Error: quantity of items should be greater or equal the size of itemset and size of the itemset greater or equal to 2!\n");
-		$itemsetSizeLimit = $_[1];
 		$self->{itemsetSize} = $_[1];
 	}
-	my @newItemsetCandidateSizeOne = ();
+	my @newCandidateItemsetSizeOne = ();
 	if($self->{messages}){
-		print "\n\nItemset size ", $self->{itemsetSize}, ", ", scalar(@itemsetCandidateSizeOne), " items ";
+		print "\nItemset size ", $self->{itemsetSize}, ", ", scalar(@candidateItemsetSizeOne), " items ";
 		print "\nProcessing... ";
 	}
-	for(my$itemsetSize=2;$itemsetSize<=$itemsetSizeLimit;$itemsetSize++){
+	for(my$itemsetSize=2;$itemsetSize<=$self->{itemsetSize};$itemsetSize++){
 		my $code=<<'CODE';
-my @itemsetCandidate=();
-my %itemsetSubset;
-for(my$k=0;$k<=$#itemsetCandidateSizeOne;$k++){
-	push @itemsetCandidate,$itemsetCandidateSizeOne[$k];
+my @candidateItemset=();
+$self->{implications}={};
+for(my$k=0;$k<=$#candidateItemsetSizeOne;$k++){
+	push @candidateItemset,$candidateItemsetSizeOne[$k];
 CODE
 		for(my$i=2;$i<=$itemsetSize;$i++){
 			$code.=<<'CODE';
-for(my$l=0;$l<=$#itemsetCandidateSizeOne;$l++){
-	if($itemsetCandidateSizeOne[$l] ~~ @itemsetCandidate){
+for(my$l=0;$l<=$#candidateItemsetSizeOne;$l++){
+	if($candidateItemsetSizeOne[$l] ~~ @candidateItemset){
 		next;
 	}
-	push @itemsetCandidate,$itemsetCandidateSizeOne[$l];
+	push @candidateItemset,$candidateItemsetSizeOne[$l];
 CODE
 		}
 		$code.=<<'CODE';
-for(my$subsetSize=0;$subsetSize<$#itemsetCandidate;$subsetSize++){
-	my @subsetLeft=();
-	my @subsetRight=();
+for(my$subsetSize=0;$subsetSize<$#candidateItemset;$subsetSize++){
+	my @antecedent=();
+	my @consequent=();
 	for(my$m=0;$m<=$subsetSize;$m++){
-		push @subsetLeft,$itemsetCandidate[$m];
+		push @antecedent,$candidateItemset[$m];
 	}
-	@subsetLeft = sort @subsetLeft;
-	@subsetRight = grep!($_~~@subsetLeft),@itemsetCandidate;
-	@subsetRight = sort @subsetRight;
-	if("@subsetRight" ~~ @{$itemsetSubset{"@subsetLeft"}}){
+	@antecedent = sort @antecedent;
+	@consequent = grep!($_~~@antecedent),@candidateItemset;
+	@consequent = sort @consequent;
+	if("@consequent" ~~ @{$self->{implications}{"@antecedent"}}){
 		next;
 	}
-	push @{$itemsetSubset{"@subsetLeft"}},"@subsetRight";
-	my @transactionsAssociation = @{$self->{itemsKeyTransactions}{$subsetLeft[0]}};
-	for(my$item=1;$item<=$#subsetLeft;$item++){
-		@transactionsAssociation = grep $_ ~~ @transactionsAssociation,@{$self->{itemsKeyTransactions}{$subsetLeft[$item]}};
-	}
-	my $supportAssociationLeft = scalar(@transactionsAssociation);
-	if($supportAssociationLeft == 0){
+	push @{$self->{implications}{"@antecedent"}},"@consequent";
+	my @transactions = @{$self->{itemsKeyTransactions}{$antecedent[0]}};
+	my $support = sprintf("%.2f",((scalar(@transactions)/$self->{totalTransactions})*100));
+	if($support < $self->{minSupport} || $support == 0){
 		next;
 	}
-	for(my$item=0;$item<=$#subsetRight;$item++){
-		@transactionsAssociation = grep $_ ~~ @transactionsAssociation,@{$self->{itemsKeyTransactions}{$subsetRight[$item]}};
+	for(my$item=1;$item<=$#antecedent;$item++){
+		@transactions = grep $_ ~~ @transactions,@{$self->{itemsKeyTransactions}{$antecedent[$item]}};
 	}
-	my $supportAssociationRight = scalar(@transactionsAssociation);
-	if($supportAssociationRight == 0){
+	my $supportAntecedent = scalar(@transactions);
+	if($supportAntecedent == 0){
 		next;
 	}
-	my $support = sprintf("%.2f",(($supportAssociationRight/$self->{totalTransactions})*100));
+	for(my$item=0;$item<=$#consequent;$item++){
+		@transactions = grep $_ ~~ @transactions,@{$self->{itemsKeyTransactions}{$consequent[$item]}};
+	}
+	my $supportConsequent = scalar(@transactions);
+	if($supportConsequent == 0){
+		next;
+	}
+	$support = sprintf("%.2f",(($supportConsequent/$self->{totalTransactions})*100));
 	if($support >= $self->{minSupport}){
-		my $confidence = sprintf("%.2f",(($supportAssociationRight/$supportAssociationLeft)*100));
+		my $confidence = sprintf("%.2f",(($supportConsequent/$supportAntecedent)*100));
 		if($confidence >= $self->{minConfidence}){
 			$self->{rule}++;
-			@{$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{rule}} = ("{ @subsetLeft } => { @subsetRight }", $support, $confidence);
-			@{$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items}} = ();
-			push @{$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items}},@subsetLeft;
-			my@items=grep!($_~~@newItemsetCandidateSizeOne),@subsetLeft;
-			push @newItemsetCandidateSizeOne,@items;
-			push @{$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items}},@subsetRight;			
-			@items=grep!($_~~@newItemsetCandidateSizeOne),@subsetRight;
-			push @newItemsetCandidateSizeOne,@items;			
+			$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{rule} = ["{ @antecedent } => { @consequent }", $support, $confidence];
+			$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items} = [@antecedent, @consequent];
+			my@items=grep!($_~~@newCandidateItemsetSizeOne),@antecedent;
+			push @newCandidateItemsetSizeOne,@items;
+			@items=grep!($_~~@newCandidateItemsetSizeOne),@consequent;
+			push @newCandidateItemsetSizeOne,@items;			
 		}
 	}
 }
 CODE
 		for(my$j=2;$j<=$itemsetSize;$j++){
 			$code.=<<'CODE';
-	pop @itemsetCandidate;
+	pop @candidateItemset;
 }
 CODE
 		}
 		$code.=<<'CODE';
-	pop @itemsetCandidate;
+	pop @candidateItemset;
 }
 CODE
-		if($itemsetSize==$itemsetSizeLimit){
+		if($itemsetSize==$self->{itemsetSize}){
 			eval $code;
 			die("Error: $@\n") if $@;
 			if($self->{messages}){
-				print "\nFrequent itemset: { @newItemsetCandidateSizeOne }, ", scalar(@newItemsetCandidateSizeOne), " items ";
+				print "\nFrequent itemset: { @newCandidateItemsetSizeOne }, ", scalar(@newCandidateItemsetSizeOne), " items ";
 			}
 			if($self->{associationRules}){
-				@{$self->{frequentItemset}}=@newItemsetCandidateSizeOne;
+				@{$self->{frequentItemset}}=@newCandidateItemsetSizeOne;
 				$self->output;
 			}
 		}
 	}
-	$itemsetSizeLimit += 1;
-	if((scalar(@newItemsetCandidateSizeOne)>=$itemsetSizeLimit)&&$self->{recursively}){
+	if((scalar(@newCandidateItemsetSizeOne)>=($self->{itemsetSize}+1))&&$self->{recursively}){
+		$self->{itemsetSize}+=1;
 		$self->{associationRules} = undef;
-		$self->association_rules_itemset_size(\@newItemsetCandidateSizeOne, $itemsetSizeLimit);
+		$self->association_rules_itemset_size(\@newCandidateItemsetSizeOne);
 	}
 }
 
@@ -182,9 +180,15 @@ sub stop{
 		print "\nStopping... ";
 		$self->output if $self->{associationRules};
 		print "\nExit? (Y/N): ";
-		my $exit = <STDIN>;
-		chomp($exit);
-		if($exit =~ /^y$/i){
+		my $answer = <STDIN>;
+		chomp($answer);
+		if($answer =~ /^y$/i){
+			print "Save data for further processing in the next run? (Y/N): ";
+			$answer = <STDIN>;
+			chomp($answer);
+			if($answer =~ /^y$/i){
+				$self->data_persistence;
+			}
 			exit;
 		}
 		else{
@@ -208,6 +212,32 @@ sub output{
 	}
 }
 
+sub data_persistence{
+	use Storable;
+	if(wantarray){
+		if(-e 'Apriori.storable'){
+			print "\nContinue processing using the data saved in the last run? (Y/N): ";
+			my$answer = <STDIN>;
+			chomp($answer);
+			if($answer =~ /^y$/i){
+				$self = retrieve('Apriori.storable');
+				unlink('Apriori.storable');
+				return @{$self->{frequentItemset}};
+			}
+			else{
+				unlink('Apriori.storable');
+				return keys(%{$self->{itemsKeyDescription}});
+			}
+		}
+		else{
+			return keys(%{$self->{itemsKeyDescription}});
+		}
+	}
+	else{
+		store($self,'Apriori.storable');
+	}
+}
+
 sub file{
 	if($self->{messages}){
 		print "\nExporting to file \"output_itemset_size_$self->{itemsetSize}.txt\"... ";
@@ -243,7 +273,8 @@ sub excel{
 	if($self->{messages}){
 		print "\nExporting to excel \"output_itemset_size_$self->{itemsetSize}.xlsx\"... ";
 	}
-	my $workbook  = Excel::Writer::XLSX->new("output_itemset_size_$self->{itemsetSize}.xlsx");
+	my $workbook  = Excel::Writer::XLSX->new("output_itemset_size_$self->{itemsetSize}.xlsx") 
+		or die("\nError: \"output_itemset_size_$self->{itemsetSize}.xlsx\" $!");
 	my $worksheet = $workbook->add_worksheet();
 	my $bold = $workbook->add_format(bold => 1);
 	my $headings = ['Rules', 'Support %', 'Confidence %'];
@@ -328,7 +359,7 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->{minConfidence}=1.55; # The minimum confidence
 
-	$apriori->{output}=1; # The output type (1 - Export to file; 2 - Export to excel)(optional)
+	$apriori->{output}=1; # The output type (1 - Export to text file delimited by tab; 2 - Export to excel file with chart)(optional)
 
 	$apriori->{messages}=1; # A value boolean to display the messages(optional)
 
@@ -343,10 +374,70 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->generate_rules; # Generate association rules to no longer meet the minimum support and confidence
 	# or
-	# $apriori->association_rules_itemset_size(3); # Generate rules from a set of items size 3, for example
+	#$apriori->association_rules_itemset_size(3); # Generate rules from a set of items size 3, for example
 
 	print "\n@{$apriori->{frequentItemset}}\n"; # Show frequent items
 
+	#output messages
+
+	3 items, 12 possible rules
+	Itemset size 2, 3 items 
+	Processing... 
+	Frequent itemset: { 101, 103, 102 }, 3 items 
+	Exporting to file "output_itemset_size_2.txt"... 
+	Itemset size 3, 3 items 
+	Processing... 
+	Frequent itemset: { 101, 102, 103 }, 3 items 
+	Exporting to file "output_itemset_size_3.txt"... 
+	101, 102, 103
+
+	#output file "output_itemset_size_2.txt"
+
+	Rules	Support %	Confidence %
+	R1	70,00	87,50
+	R2	70,00	87,50
+	R3	70,00	77,78
+	R4	80,00	88,89
+	R5	70,00	77,78
+	R6	80,00	88,89
+
+	Rule R1: { 101 } => { 103 }
+	Support: 70,00 %
+	Confidence: 87,50 %
+	Items:
+	101 MILK
+	103 CEREAL
+
+	to be continued...
+
+	#output file "output_itemset_size_3.txt"
+
+	Rules	Support %	Confidence %
+	R7	60,00	75,00
+	R8	60,00	85,71
+	R9	60,00	85,71
+	R10	60,00	66,67
+	R11	60,00	75,00
+	R12	60,00	66,67
+
+	Rule R7: { 101 } => { 102, 103 }
+	Support: 60,00 %
+	Confidence: 75,00 %
+	Items:
+	101 MILK
+	102 BREAD
+	103 CEREAL
+
+	Rule R8: { 101, 103 } => { 102 }
+	Support: 60,00 %
+	Confidence: 85,71 %
+	Items:
+	101 MILK
+	103 CEREAL
+	102 BREAD
+
+	to be continued...
+
 	# or from a database
 
 	# CREATE TABLE dimension_product(
@@ -354,13 +445,12 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	# 	product_alternate_key INTEGER NOT NULL,
 	# 	product_name TEXT NOT NULL,
 	# 	price REAL NOT NULL
-	#  // ...
+	#	// ...
 	# );
 	#
 	# INSERT INTO dimension_product VALUES(1,101,'MILK',10.00);
 	# INSERT INTO dimension_product VALUES(2,102,'BREAD',10.00);
 	# INSERT INTO dimension_product VALUES(3,103,'CEREAL',10.00);
-	# 
 	# // ...
 	# 
 	# CREATE TABLE fact_sales(
@@ -368,7 +458,7 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	# 	sales_order_line_number INTEGER NOT NULL,
 	# 	product_key INTEGER NOT NULL,
 	# 	quantity INTEGER NOT NULL,
-	#  // ...
+	#	// ...
 	# 	PRIMARY KEY(sales_order_number, sales_order_line_number),
 	# 	FOREIGN KEY(product_key) REFERENCES dimension_product(product_key)
 	# );
@@ -399,6 +489,7 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 	# INSERT INTO fact_sales VALUES(1110,1,1,1);
 	# INSERT INTO fact_sales VALUES(1110,2,2,1);
 	# INSERT INTO fact_sales VALUES(1110,3,3,1);
+	#//...
 
 	use DBD::SQLite;
 	use Data::Mining::Apriori;
@@ -426,16 +517,12 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->{messages}=1;
 
-	my $support = int((($apriori->{totalTransactions}/100)*$apriori->{minSupport}));
-
 	$sql = qq~
-	SELECT dp.product_alternate_key, dp.product_name, COUNT(*)
+	SELECT dp.product_alternate_key, dp.product_name
 	FROM dimension_product dp
 	JOIN fact_sales fs ON
 	dp.product_key = fs.product_key
 	/* WHERE ... */
-	GROUP BY dp.product_alternate_key, dp.product_name
-	HAVING COUNT(*) >= $support
 	~;
 
 	$query = $db->prepare($sql);
@@ -462,7 +549,7 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->generate_rules;
 	# or
-	# $apriori->association_rules_itemset_size(3);
+	#$apriori->association_rules_itemset_size(3);
 
 	print "\n@{$apriori->{frequentItemset}}\n";
 
@@ -490,13 +577,9 @@ The output type (optional):
 
 =over 4
 
-=item 1
+=item 1- Text file delimited by tab;
 
-- Text file;
-
-=item 2
-
-- Excel file.
+=item 2- Excel file with chart.
 
 =back
 
@@ -524,6 +607,26 @@ Frequent itemset.
 
 A value boolean to generate association rules until no set of items meets the minimum support or minimum confidence.
 
+=head2 associationRules
+
+a data structure to store the name of the rule, key items, implication, support and confidence.
+
+	$self->{associationRules} = {
+								  '1' => {
+										   'R1' => {
+													 'items' => [
+																  '101',
+																  '103'
+																],
+													 'rule' => [
+																 '{ 101 } => { 103 }',
+																 '70.00',
+																 '87.50'
+															   ]
+												   }
+										 },
+									# to be continued...
+
 =head1 METHODS
 
 =head2 new
@@ -540,9 +643,7 @@ Generates association rules by size set of items. Accepts the following argument
 
 =over 4
 
-=item *
-
-An integer.
+=item * An integer.
 
 Representing the size of the set of items.
 
